###########################################################
## This is an R file that forms part of the project ".."
## by Genaro Sucarrat and ...
##
## First created 18 February 2014, Oslo.
##
## -- Part 1: load libraries, compile code
## .ACLRECURSION1C          #compiled C code
##
## -- Part 2: acl model
## TO BE DONE BEFORE PUBLICATION?: PREDICT
## aclSim             #simulate
## aclRecursion1      #recursion (1st order)
## aclLogl            #log-likelihood
## acl                #estimation
##
## -- Part 3: S3 methods
## coef.acl           #S3 methods
## fitted.acl
## logLik.acl
## print.acl
## predict.acl
## residuals.acl
## summary.acl
## vcov.acl
##
###########################################################

##---------------------------------------------------------
## 1 LOAD LIBRARIES, COMPILE C-CODE
##---------------------------------------------------------

library(inline)
library(compiler)
library(zoo)

#.ACLRECURSION1C:
mysig <- signature(iStart="integer", iEnd="integer", rho1="numeric",
  zeta1="numeric", innov="numeric", vIadj="numeric", vS="numeric",
  pi1t="numeric", ht="numeric")
mycode <- "
  for (int i=*iStart; i < *iEnd; i++) {
    ht[i] = innov[i] + *rho1 * vS[i-1] + *zeta1 * ht[i-1];
    pi1t[i] = 1/( 1 + exp(-ht[i]) );
    vS[i] = (vIadj[i]-pi1t[i])/sqrt( pi1t[i] * (1-pi1t[i]) );
  }
"
.ACLRECURSION1C <- cfunction(mysig, mycode, convention=".C") #compile
rm(mysig, mycode)

##---------------------------------------------------------
## 2 ACL MODEL
##---------------------------------------------------------

#####################
aclSim <- function(n, constant=0.15, ma=0.1, ar=0.8,
  xreg=NULL, initial.values=list(s=NULL, h=NULL, xreg=NULL),
  verbose=FALSE)
{
#############################################################
#NEXT THING TO DO: RECURSION WHEN AR/MA ORDERS GREATER THAN 1
#############################################################

  #chk args:
  #-chk stability of ar pars?
  #-as.list(initial.values)

  #orders:
  q.order <- length(ma)
  p.order <- length(ar)
  maxpq <- max(1, q.order, p.order)
  nplussMaxpq <- n+maxpq

  #xreg:
  if(is.null(xreg)){
    result.index <- 1:n
    xreg <- rep(0,nplussMaxpq)
    xregMean <- 0
  }else{
    xreg <- as.zoo(xreg)
    result.index <- index(xreg)
    xreg <- coredata(xreg)
    if(is.null(initial.values$xreg)){
      xregMean <- mean(xreg)
      xreg.init <- rep(xregMean,maxpq)
    }else{
      xreg.init <- initial.values$xreg
    }
    xreg <- c(xreg.init, xreg)
  }

  #ht variable:
  ht <- rep(0,nplussMaxpq)
  if(is.null(initial.values$h)){
    ht[1:maxpq] <- (constant+xregMean)/(1-sum(ar))
  }else{
    ht[1:maxpq] <- initial.values$h
  }

  #pi1, vS and vI variables:
  pi1 <- 1/(1+exp(-ht))
  vS <- rep(0,nplussMaxpq)
  vI <- rep(0,nplussMaxpq)
  vI[1:maxpq] <- pi1[1:maxpq]
  vItmp <- c(rep(0,maxpq), runif(n)-1)

  #recursion:
  innov <- constant + xreg
  if(maxpq <= 1){
    for(i in c(maxpq+1):nplussMaxpq){
      ht[i] <- innov[i] + ma[1]*vS[i-1] + ar[1]*ht[i-1]
      pi1[i] <- 1/(1+exp(-ht[i]))
      vI[i] <- as.numeric(vItmp[i]+pi1[i] > 0)
      vS[i] <- (vI[i]-pi1[i])/sqrt(pi1[i]*(1-pi1[i]))
    }
  }

  #output:
  if(verbose){
    result <- cbind(vI, pi1, ht, vS)[-1,]
    colnames(result) <- c("I", "pi1", "h", "s")
  }else{
    result <- vI[-1]
  }
  result <- zoo(result, order.by=result.index)
  return(result)
} #end aclSim

##compile aclSim:
aclSim <- cmpfun(aclSim)

#####################
aclRecursion1 <- function(pars, aux)
{
  #make vectors:
  if(aux$xreg.k == 0){
    innov <- pars[1]
    innovMean <- mean(innov)
    innov <- rep(innov, max(aux$n+1,aux$nplussMaxpq))
  }else{
    innov <- pars[1] + aux$xreg %*% pars[aux$xreg.indx]
    innovMean <- mean(innov)
    innov <- c(rep(innovMean, max(1,aux$maxpq)), innov)
  }
  ht <- aux$zerosaux
  ht[1:max(1,aux$maxpq)] <- innovMean/(1-sum(pars[aux$ar.indx]))
  pi1t <- 1/(1+exp(-ht))
  vS <- aux$zerosaux
  vIadj <- c(pi1t[1:max(1,aux$maxpq)], aux$vI)

  #recursion:
  if(aux$maxpq > 0){
    if(aux$c.code){
      iStart <- max(2,aux$maxpq+1) - 1
      iEnd <- max(aux$nplussMaxpq, aux$n+1)
      if(aux$ar > 0){ zeta1 <- pars[3] }else{ zeta1 <- 0 }
      tmp <- .ACLRECURSION1C(as.integer(iStart), as.integer(iEnd),
        as.numeric(pars[2]), as.numeric(zeta1), as.numeric(innov),
        as.numeric(vIadj), as.numeric(vS), as.numeric(pi1t),
        as.numeric(ht))
      #tmp$vS[npluss1] <- NA
    }else{
      for(i in max(2,aux$maxpq+1):max(aux$nplussMaxpq, aux$n+1)){
        ht[i] <- innov[i] + pars[2]*vS[i-1]
        if(aux$ar > 0){
          ht[i] <- ht[i] + pars[aux$ar.indx]*ht[i-1]
        }
        pi1t[i] <- 1/(1+exp(-ht[i]))
        vS[i] <- (vIadj[i]-pi1t[i])/sqrt(pi1t[i]*(1-pi1t[i]))
      } #end for loop
    } #end if(c.code)
  }else{
    ht <- innov
    pi1t <- 1/(1+exp(-ht))
    vS <- rep(NA, length(innov))
  } #end if(maxpq > 0)

  #output:
  if(aux$verboseRecursion){
    if(aux$maxpq > 0 && aux$c.code==TRUE){
        result <- cbind(tmp$pi1t, tmp$ht, tmp$vS, tmp$vIadj)
    }else{
      result <- cbind(pi1t, ht, vS, vIadj)
    } #end
    result <- result[-c(1:max(1,aux$maxpq)),]
    colnames(result) <- c("pi1", "h", "s", "I")
  }else{
    if(aux$maxpq > 0 && aux$c.code==TRUE){
      result <- tmp$pi1t
    }else{
      result <- pi1t
    } #end
    result <- result[-c(1:max(1,aux$maxpq))]
  } #end if(verboseRecursion)
  return(result)
} #end aclRecursion1

#####################
aclLogl <- function(pars, aux)
{
  #begin code:
  if( any(is.na(pars)) || any(pars<=aux$lower) || any(pars>=aux$upper) ){
    chk.conds <- FALSE
  }else{
    chk.conds <- TRUE
  }

  if(chk.conds){
    pi1that <- aclRecursion1(pars, aux)
    logl <- sum( aux$vI*log(pi1that) + aux$vI0*log(1-pi1that) )
    if(is.nan(logl) || is.na(logl) || abs(logl) == Inf){
      logl <- aux$logl.penalty
    }
  }else{
    logl <- aux$logl.penalty
  }
  return(logl)
} #end aclLogl

#####################
acl <- function(x, ma=1, ar=1, xreg=NULL,
  backcast.values=list(s=NULL, h=NULL, xreg=NULL),
  initial.values=NULL, lower=NULL, upper=NULL,
  nlminb.control=list(), vcov=TRUE, verbose=TRUE,
  logl.penalty=NULL, solve.tol=.Machine$double.eps,
  c.code=TRUE, store.aux=TRUE)
{
  #check arguments:
  #stop(ma==0 && ar > 0) "ma order must be greater than 0 if ar order is greater than 0"
  #check ar
  #check ma
  if(!is.null(xreg)) xreg <- cbind(coredata(xreg))

  #aux list:
  aux <- list()

  #zoo:
  vI <- as.zoo(x)
  vI <- na.trim(vI)
  vI.index <- index(vI)
  vI <- coredata(vI)
  aux$vI <- vI
  aux$vI.index <- vI.index
  aux$vI0 <- (1-vI)
  aux$n <- length(vI)

  #orders and initial values:
  aux$maxpq <- max(ma,ar)
  aux$nplussMaxpq <- aux$n + aux$maxpq
  aux$ma <- ma
  aux$ar <- ar
  if(ma>0){ aux$ma.indx <- 2:c(ma+1) }else{ aux$ma.indx <- 0 }
  if(ar>0){
    aux$ar.indx <- c(max(aux$ma.indx)+1):c(max(aux$ma.indx)+ar)
  }else{
    aux$ar.indx <- 0
  }
  if(is.null(xreg)){
    aux$xreg.k <- 0
  }else{
    aux$xreg <- cbind(xreg)
    aux$xreg.k <- ncol(aux$xreg)
    aux$xreg.indx <- c(max(1,aux$ma.indx,aux$ar.indx)+1):c(max(1,aux$ma.indx,aux$ar.indx)+aux$xreg.k)
  }
  if(is.null(initial.values)){
    initial.values <- c(0.15, rep(0.1/ma, ma), rep(0.8/ar, ar))
    if(!is.null(xreg)){
      initial.values <- c(initial.values, rep(0.01, aux$xreg.k))
    }
    aux$initial.values <- initial.values
  }

  #upper bounds:
  if(is.null(upper)){
    upper <- c(Inf, rep(Inf,ma), rep(1-.Machine$double.eps,ar))
    if(!is.null(xreg)){
      upper <- c(upper, rep(Inf, aux$xreg.k))
    }
    aux$upper <- upper
  }
  #instead?:
  #upper <- c(Inf, rep(Inf,ma), rep(1-.Machine$double.eps,ar),
  #  rep(Inf, aux$xreg.k))

  #lower bounds:
  if(is.null(lower)){
    lower <- c(-Inf, rep(-Inf,ma), rep(.Machine$double.eps-1,ar))
    if(!is.null(xreg)){
      lower <- c(lower, rep(-Inf, aux$xreg.k))
    }
    aux$lower <- lower
  }
  #instead?:
  #lower <- c(-Inf, rep(-Inf,ma), rep(.Machine$double.eps-1,ar),
  #   rep(-Inf, aux$xreg.k))


  #misc:
  aux$c.code <- c.code
  aux$solve.tol <- solve.tol
  aux$verboseRecursion <- FALSE
  aux$zerosaux <- rep(0,max(aux$nplussMaxpq, aux$n+1))
  if(is.null(logl.penalty)){
    aux$logl.penalty <- aclLogl(initial.values, aux)
  }else{
    aux$logl.penalty <- logl.penalty
  }

  #estimate:
  objective.f <- function(pars, x=aux){ -aclLogl(pars,x) }
  est <- nlminb(initial.values, objective.f, lower=lower,
    upper=upper, control=nlminb.control)
  est <- c(list(date=date()), est)

  #regressor names:
  namesAcl <- "constant"
  if(aux$ma > 0){
    namesAcl <- c(namesAcl, paste("ma",1:aux$ma,sep=""))
  }
  if(aux$ar > 0){
    namesAcl <- c(namesAcl, paste("ar",1:aux$ar,sep=""))
  }
  if(aux$xreg.k > 0){
    namesAcl <- c(namesAcl, paste("xreg",1:aux$xreg.k,sep=""))
  }

  #vcov matrix:
  if(vcov){
    hessianAcl <- optimHess(as.numeric(est$par), objective.f)
    colnames(hessianAcl) <- namesAcl
    rownames(hessianAcl) <- namesAcl
    vcovmat <- solve(hessianAcl, tol=solve.tol)
    est <- c(list(hessian=hessianAcl, vcov=vcovmat), est)
  }

  #out:
  est$objective <- -est$objective
  names(est$par) <- namesAcl
  if(store.aux==TRUE){
    est <- c(list(aux=aux),est)
  }
  class(est) <- "acl"
  return(est)
} #end acl

##---------------------------------------------------------
## 3 S3 METHODS
##---------------------------------------------------------

##################
coef.acl <- function(object, ...)
{
  return(object$par)
} #end coef.acl

################## fitted values (wrapper to aclRecursion1)
fitted.acl <- function(object, verbose=FALSE, ...)
{
  aux <- object$aux
  aux$verboseRecursion <- verbose
  out <- aclRecursion1(as.numeric(object$par), aux)
  out <- zoo(out, order.by=aux$vI.index)
  return(out)
} #end fitted.acl

################## return log-likelihood
logLik.acl <- function(object, ...)
{
  out <- object$objective
  attr(out, "nobs") <- length(object$aux$n)
  attr(out, "df") <- length(object$par)
  class(out) <- "logLik"
  return(out)
} #end logLik.acl

################## forecast
#predict.acl <- function(object, n.ahead=1,
#  initial.values=NULL, n.sim=10000, verbose=FALSE, ...)
#{
#return(out)
#} #end predict.acl

################## summarise output
print.acl <- function(x, ...)
{
  vcovmat <- vcov(x)
  out1 <- rbind(x$par, sqrt(diag(vcovmat)))
  rownames(out1) <- c("Estimate:", "Std. Error:")
  out2 <- rbind(x$objective)
  rownames(out2) <- c("Log-likelihood:")
  colnames(out2) <- ""

  cat("Date:", x$date, "\n")
  cat("Message (nlminb):", x$message, "\n")
  cat("\n")
  cat("Coefficients:\n")
  print(out1)
  print(out2)
} #end print.acl

################## residual values (wrapper to ..)
residuals.acl <- function(object, ...)
{
  aux <- object$aux
  aux$verboseRecursion <- TRUE
  out <- aclRecursion1(as.numeric(object$par), aux)
  out <- zoo(out[,"s"], order.by=aux$vI.index)
  return(out)
} #end residuals.acl

################## summarise output
summary.acl <- function(object, verbose=FALSE, ...)
{
  out <- NULL
  if(verbose){
    out <- list(vcov=object$vcov,
      initial.values=object$aux$initial.values,
      upper=object$aux$upper, lower=object$aux$lower)
  }
  out <- c(out, list(date=object$date, par=object$par,
    objective=object$objective, iterations=object$iterations,
    message=object$message))
  return(out)
} #end summary.acl

################## return variance-covariance matrix
vcov.acl <- function(object, ...)
{
  if(is.null(object$vcov)){
    if(is.null(object$aux)){
      cat("Auxiliary list 'aux' not stored, vcov cannot be extracted\n")
      cat("Re-estimate with vcov = TRUE or store.aux = TRUE \n")
      vcovmat <- NULL
    }else{
      objective.f <- function(pars, x=object$aux){ -aclLogl(pars,x) }
      hessianAcl <- optimHess(as.numeric(object$par), objective.f)
      colnames(hessianAcl) <- names(object$par)
      rownames(hessianAcl) <- names(object$par)
      vcovmat <- solve(hessianAcl, tol=object$solve.tol)
    }
  }else{
    vcovmat <- object$vcov
  }
  return(vcovmat)
} #end vcov.acl
