###########################################################
## This is an R file that forms part of the project
## "Estimation of log-GARCH models in the presence of zero
## returns" by Genaro Sucarrat and Alvaro Escribano.
##
## The file contains the code that undertakes the analyses
## contained in Section 4.1, and was first created 19
## August 2013, Rygge (Norway).
##
## CONTENTS:
##
## 1 INITIATE
## 2 AUXILIARY FUNCTIONS
## 3 APPLE
## 4 EKORNES
## 5 EUR/USD
## 6 SP500
## 7 OILPRICE
## 8 GOLD
## 9 MAKE DESCRIPTIVE STATISTICS AND ESTIMATES TABLE
## 10 MAKE FIGURES
## 11 CHECK WHETHER MEAN-CORRECTION REMOVES ARCH
##
###########################################################


###########################################################
## 1 INITIATE
###########################################################

##remove everything in environment:
rm(list=ls())

##set working directory:
#setwd("C:/Users/sucarrat/Documents/projects/zerologgarch/analysis/20160912")
setwd(choose.dir())

##load R libraries of functions:
#source("C:/Users/sucarrat/Documents/R/gs/lgarch/20140901build/20140901lgarch.Rpackage.R")
source("20140901lgarch.Rpackage.R") #lgarch version 0.5
library(xtable)


###########################################################
##2 AUXILIARY FUNCTIONS
###########################################################

##rounding function:
ground <- function(x, digits=0, eps=.Machine$double.eps,
  as.character=FALSE)
{
  xpos <- as.numeric(x>0)*(x + eps)
  xneg <- as.numeric(x<0)*(x - eps)
  x <- xpos + xneg
  if(as.character){
    result <- format(round(x, digits=digits), nsmall=digits)
  }else{
    result <- round(x, digits=digits)
  }
  return(result)
} #end ground

##sample kurtosis
gkurtosis <- function(y)
{
  m4 <- mean((y-mean(y))^4)
  kurt <- m4/(sd(y)^4)
  return(kurt)
} #end gkurtosis

##sample skewness
gskewness <- function(y)
{
  m3 <- mean((y-mean(y))^3)
  skew <- m3/(sd(y)^3)
  return(skew)
} #end gskewness

##perform k Ljung-Box tests:
gBox.test <- function(x, lag=1:15, type=c("Ljung-Box", "Box-Pierce"),
  fitdf=0)
{
  ##initialise:
  type <- match.arg(type)
  x <- coredata(as.zoo(x))
  xIsNA <- which( is.na(x) )

  ##construct out-matrix:
  out <- matrix(NA, length(lag), 4)
  colnames(out) <- c("AutoCorr", "Q-stat", "p-value", "n")
  rownames(out) <- lag

  ##compute autocorrelations and sample sizes (i.e. n):
  for(i in 1:max(lag)){
    xlagged <- glag(x, k=i, pad=TRUE, pad.value=NA)
    xlaggedIsNA <- which( is.na(xlagged) )
    whereNAs <- union(xIsNA,xlaggedIsNA)
    xadj <- x[-whereNAs]
    xlagged <- xlagged[-whereNAs]
    out[i,1] <- cor(xadj,xlagged)
    out[i,4] <- length(xadj)
  } #close for-loop

  ##compute test-statistic and p-values:
  outRow <- 0
  for(i in lag){
    n <- out[i,4]
    if(type=="Box-Pierce"){
      testStat <- n*sum(out[1:i,1]^2)
    }else{  ##Ljung-Box:
      testStat <- n*(n+2)*sum(1/seq.int(n-1, n-i) * out[1:i,1]^2)
    }
    outRow <- outRow + 1
    out[outRow,2] <- testStat
    out[outRow,3] <- 1-pchisq(testStat, i-fitdf)
  }
  return(out)
} ## end gBox.test


###########################################################
## 3 APPLE
###########################################################

mapple <- read.csv("table_apple.csv")
apple <- zoo(mapple[,"Adj.Close"], order.by=as.Date(mapple[,1], "%Y-%m-%d"))
Dlapple <- diff(log(apple))*100

##zeros replaced by small non-zero value (0-adj):
Dlapple.where.zeros <- which( Dlapple == 0)
Dlapple.zero.eps <- min(abs(Dlapple[-Dlapple.where.zeros]))
Dlappleadj <- Dlapple
Dlappleadj[Dlapple.where.zeros] <- Dlapple.zero.eps

##0-adj model:
Dlapple.Mod01 <- lgarch(Dlappleadj, method="ml",
    mean.correction=FALSE)
Dlapple.Mod01Fit <- fitted(Dlapple.Mod01, verbose=TRUE)

##0-adj model w/asymmetry:
DlappleAsym <- log(Dlappleadj^2)*(Dlapple < 0)
DlappleAsym <- glag(DlappleAsym, pad.value=0)
Dlapple.Mod03 <- lgarch(Dlappleadj, xreg=DlappleAsym,
  method="ml", mean.correction=FALSE)
Dlapple.Mod03Fit <- fitted(Dlapple.Mod03, verbose=TRUE)

##zeros not replaced (Algo):
Dlapple.Mod02 <- lgarch(Dlapple, method="ml",
    mean.correction=FALSE)
Dlapple.Mod02Fit <- fitted(Dlapple.Mod02, verbose=TRUE)
Dlapple.Mod02zhat <- Dlapple.Mod02Fit[,"zhat"]
Dlapple.Mod02zhat[ which( Dlapple.Mod02zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/asymmetry:
Dlapple.Mod04 <- lgarch(Dlapple, xreg=DlappleAsym,
  method="ml", mean.correction=FALSE)
Dlapple.Mod04Fit <- fitted(Dlapple.Mod04, verbose=TRUE)
Dlapple.Mod04zhat <- Dlapple.Mod04Fit[,"zhat"]
Dlapple.Mod04zhat[ which( Dlapple.Mod04zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/extended asymmetry:
DlappleAsymEx <- (Dlapple < 0)
DlappleAsymEx <- glag(DlappleAsymEx, pad.value=0)
Dlapple.Mod06 <- lgarch(Dlapple, xreg=cbind(DlappleAsym,DlappleAsymEx),
  method="ml", mean.correction=FALSE)
Dlapple.Mod06Fit <- fitted(Dlapple.Mod06, verbose=TRUE)
Dlapple.Mod06zhat <- Dlapple.Mod06Fit[,"zhat"]
Dlapple.Mod06zhat[ which( Dlapple.Mod06zhat==0 ) ] <- NA

##latex code:
Dlapple.latex <- paste("Apple (10 Sep. 1984 -- 23 Aug. 2013): & ",
  ground(var(Dlapple), digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dlapple),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox.test(Dlapple^2)[1,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dlapple^2)[1,2], digits=2, as.character=TRUE),
  "}$ & ",
  "$\\underset{[", ground(gBox.test(Dlapple^2)[2,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dlapple^2)[2,2], digits=2, as.character=TRUE),
  "}$ & ",
  length(Dlapple), " & ",
  length(Dlapple.where.zeros), " & ",
  ground(length(Dlapple.where.zeros)/length(Dlapple), digits=3,
  as.character=TRUE), " \\", sep="")
Dlapple.latexMod01 <- paste("Apple: & & 0-adj & ",
  ground(as.numeric(Dlapple.Mod01$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod01$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod01))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod01$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod01))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlapple.Mod01Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlapple.Mod01Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dlapple.latexMod02 <- paste(" & & Algo & ",
  ground(as.numeric(Dlapple.Mod02$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod02$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod02))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod02$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod02))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlapple.Mod02zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlapple.Mod02zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")
Dlapple.latexMod03 <- paste("Apple: & & 0-adj & ",
  ground(as.numeric(Dlapple.Mod03$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod03$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod03))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod03$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod03))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod03$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod03))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlapple.Mod03Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlapple.Mod03Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dlapple.latexMod04 <- paste(" & & Algo & ",
  ground(as.numeric(Dlapple.Mod04$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod04$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod04))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod04$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod04))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlapple.Mod04$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlapple.Mod04))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlapple.Mod04zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlapple.Mod04zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")


###########################################################
## 4 EKORNES
###########################################################

mekornes <- read.csv("table_ekornes.csv")
ekornes <- zoo(mekornes[,"Adj.Close"], order.by=as.Date(mekornes[,1], "%Y-%m-%d"))
Dlekornes <- diff(log(ekornes))*100

##zeros replaced by small non-zero value (0-adj):
Dlekornes.where.zeros <- which( Dlekornes == 0)
Dlekornes.zero.eps <- min(abs(Dlekornes[-Dlekornes.where.zeros]))
Dlekornesadj <- Dlekornes
Dlekornesadj[Dlekornes.where.zeros] <- Dlekornes.zero.eps

##0-adj model:
Dlekornes.Mod01 <- lgarch(Dlekornesadj, method="ml",
    mean.correction=FALSE)
Dlekornes.Mod01Fit <- fitted(Dlekornes.Mod01, verbose=TRUE)

##0-adj model w/asymmetry:
DlekornesAsym <- log(Dlekornesadj^2)*(Dlekornes < 0)
DlekornesAsym <- glag(DlekornesAsym, pad.value=0)
Dlekornes.Mod03 <- lgarch(Dlekornesadj, xreg=DlekornesAsym,
  method="ml", mean.correction=FALSE)
Dlekornes.Mod03Fit <- fitted(Dlekornes.Mod03, verbose=TRUE)

##zeros not replaced (Algo):
Dlekornes.Mod02 <- lgarch(Dlekornes, method="ml",
    mean.correction=FALSE)
Dlekornes.Mod02Fit <- fitted(Dlekornes.Mod02, verbose=TRUE)
Dlekornes.Mod02zhat <- Dlekornes.Mod02Fit[,"zhat"]
Dlekornes.Mod02zhat[ which( Dlekornes.Mod02zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/asymmetry:
Dlekornes.Mod04 <- lgarch(Dlekornes, xreg=DlekornesAsym,
  method="ml", mean.correction=FALSE)
Dlekornes.Mod04Fit <- fitted(Dlekornes.Mod04, verbose=TRUE)
Dlekornes.Mod04zhat <- Dlekornes.Mod04Fit[,"zhat"]
Dlekornes.Mod04zhat[ which( Dlekornes.Mod04zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/extended asymmetry:
DlekornesAsymEx <- (Dlekornes < 0)
DlekornesAsymEx <- glag(DlekornesAsymEx, pad.value=0)
Dlekornes.Mod06 <- lgarch(Dlekornes, xreg=cbind(DlekornesAsym,DlekornesAsymEx),
  method="ml", mean.correction=FALSE)
Dlekornes.Mod06Fit <- fitted(Dlekornes.Mod06, verbose=TRUE)
Dlekornes.Mod06zhat <- Dlekornes.Mod06Fit[,"zhat"]
Dlekornes.Mod06zhat[ which( Dlekornes.Mod06zhat==0 ) ] <- NA

##latex code:
Dlekornes.latex <- paste("Apple (10 Sep. 1984 -- 23 Aug. 2013): & ",
  ground(var(Dlekornes), digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dlekornes),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox.test(Dlekornes^2)[1,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dlekornes^2)[1,2], digits=2, as.character=TRUE),
  "}$ & ",
  "$\\underset{[", ground(gBox.test(Dlekornes^2)[2,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dlekornes^2)[2,2], digits=2, as.character=TRUE),
  "}$ & ",
  length(Dlekornes), " & ",
  length(Dlekornes.where.zeros), " & ",
  ground(length(Dlekornes.where.zeros)/length(Dlekornes), digits=3,
  as.character=TRUE), " \\", sep="")
Dlekornes.latexMod01 <- paste("Ekornes: & & 0-adj & ",
  ground(as.numeric(Dlekornes.Mod01$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod01$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod01))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod01$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod01))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlekornes.Mod01Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlekornes.Mod01Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dlekornes.latexMod02 <- paste(" & & Algo & ",
  ground(as.numeric(Dlekornes.Mod02$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod02$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod02))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod02$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod02))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlekornes.Mod02zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlekornes.Mod02zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")
Dlekornes.latexMod03 <- paste("Ekornes: & & 0-adj & ",
  ground(as.numeric(Dlekornes.Mod03$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod03$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod03))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod03$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod03))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod03$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod03))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlekornes.Mod03Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlekornes.Mod03Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dlekornes.latexMod04 <- paste(" & & Algo & ",
  ground(as.numeric(Dlekornes.Mod04$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod04$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod04))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod04$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod04))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlekornes.Mod04$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlekornes.Mod04))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlekornes.Mod04zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlekornes.Mod04zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")


###########################################################
## 5 EUR/USD
###########################################################

mfx <- read.csv("eurofxref-hist.csv")
eurusd <- zoo(mfx[,"USD"], order.by=as.Date(mfx[,1], "%Y-%m-%d"))
FWZdate <- which( index(eurusd)=="2012-01-18" )
eurusd <- eurusd[1:FWZdate]
Dleurusd <- diff(log(eurusd))*100

##zeros replaced by small non-zero value (0-adj):
Dleurusd.where.zeros <- which( Dleurusd == 0)
Dleurusd.zero.eps <- min(abs(Dleurusd[-Dleurusd.where.zeros]))
Dleurusdadj <- Dleurusd
Dleurusdadj[Dleurusd.where.zeros] <- Dleurusd.zero.eps

##0-adj model:
Dleurusd.Mod01 <- lgarch(Dleurusdadj, method="ml",
    mean.correction=FALSE)
Dleurusd.Mod01Fit <- fitted(Dleurusd.Mod01, verbose=TRUE)

##0-adj model w/asymmetry:
DleurusdAsym <- log(Dleurusdadj^2)*(Dleurusd < 0)
DleurusdAsym <- glag(DleurusdAsym, pad.value=0)
Dleurusd.Mod03 <- lgarch(Dleurusdadj, xreg=DleurusdAsym,
  method="ml", mean.correction=FALSE)
Dleurusd.Mod03Fit <- fitted(Dleurusd.Mod03, verbose=TRUE)

##zeros not replaced (Algo):
Dleurusd.Mod02 <- lgarch(Dleurusd, method="ml",
    mean.correction=FALSE)
Dleurusd.Mod02Fit <- fitted(Dleurusd.Mod02, verbose=TRUE)
Dleurusd.Mod02zhat <- Dleurusd.Mod02Fit[,"zhat"]
Dleurusd.Mod02zhat[ which( Dleurusd.Mod02zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/asymmetry:
Dleurusd.Mod04 <- lgarch(Dleurusd, xreg=DleurusdAsym,
  method="ml", mean.correction=FALSE)
Dleurusd.Mod04Fit <- fitted(Dleurusd.Mod04, verbose=TRUE)
Dleurusd.Mod04zhat <- Dleurusd.Mod04Fit[,"zhat"]
Dleurusd.Mod04zhat[ which( Dleurusd.Mod04zhat==0 ) ] <- NA

##checking stability of model 04:
Prz <- mean( Dleurusd.Mod04zhat[ -which(is.na(Dleurusd.Mod04zhat)) ] > 0 )
a1 <- coef(Dleurusd.Mod04)["arch1"]
b1 <- coef(Dleurusd.Mod04)["garch1"]
c1 <- coef(Dleurusd.Mod04)["xreg1"]
abs(a1+b1)^Prz * abs(a1+b1-c1)^(1-Prz)

##zeros not replaced (Algo) w/extended asymmetry:
DleurusdAsymEx <- (Dleurusd < 0)
DleurusdAsymEx <- glag(DleurusdAsymEx, pad.value=0)
Dleurusd.Mod06 <- lgarch(Dleurusd, xreg=cbind(DleurusdAsym,DleurusdAsymEx),
  method="ml", mean.correction=FALSE)
Dleurusd.Mod06Fit <- fitted(Dleurusd.Mod06, verbose=TRUE)
Dleurusd.Mod06zhat <- Dleurusd.Mod06Fit[,"zhat"]
Dleurusd.Mod06zhat[ which( Dleurusd.Mod06zhat==0 ) ] <- NA

##latex code:
Dleurusd.latex <- paste("Apple (10 Sep. 1984 -- 23 Aug. 2013): & ",
  ground(var(Dleurusd), digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dleurusd),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox.test(Dleurusd^2)[1,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dleurusd^2)[1,2], digits=2, as.character=TRUE),
  "}$ & ",
  "$\\underset{[", ground(gBox.test(Dleurusd^2)[2,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dleurusd^2)[2,2], digits=2, as.character=TRUE),
  "}$ & ",
  length(Dleurusd), " & ",
  length(Dleurusd.where.zeros), " & ",
  ground(length(Dleurusd.where.zeros)/length(Dleurusd), digits=3,
  as.character=TRUE), " \\", sep="")
Dleurusd.latexMod01 <- paste("EUR/USD: & & 0-adj & ",
  ground(as.numeric(Dleurusd.Mod01$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod01$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod01))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod01$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod01))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dleurusd.Mod01Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dleurusd.Mod01Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dleurusd.latexMod02 <- paste(" & & Algo & ",
  ground(as.numeric(Dleurusd.Mod02$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod02$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod02))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod02$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod02))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dleurusd.Mod02zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dleurusd.Mod02zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")
Dleurusd.latexMod03 <- paste("EUR/USD: & & 0-adj & ",
  ground(as.numeric(Dleurusd.Mod03$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod03$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod03))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod03$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod03))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod03$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod03))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dleurusd.Mod03Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dleurusd.Mod03Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dleurusd.latexMod04 <- paste(" & & Algo & ",
  ground(as.numeric(Dleurusd.Mod04$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod04$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod04))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod04$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod04))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dleurusd.Mod04$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dleurusd.Mod04))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dleurusd.Mod04zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dleurusd.Mod04zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")


###########################################################
## 6 SP500
###########################################################

mSP500 <- read.csv("table_sp500.csv")
sp500 <- zoo(mSP500[,"Adj.Close"], order.by=as.Date(mSP500[,1], "%Y-%m-%d"))
Dlsp500 <- diff(log(sp500))*100
t1 <- which(index(Dlsp500) == "1999-01-04")
t2 <- which(index(Dlsp500) == "2013-08-23")
Dlsp500 <- Dlsp500[t1:t2]

##zeros replaced by small non-zero value (0-adj):
Dlsp500.where.zeros <- which( Dlsp500 == 0)
Dlsp500.zero.eps <- min(abs(Dlsp500[-Dlsp500.where.zeros]))
Dlsp500adj <- Dlsp500
Dlsp500adj[Dlsp500.where.zeros] <- Dlsp500.zero.eps

##0-adj model:
Dlsp500.Mod01 <- lgarch(Dlsp500adj, method="ml",
    mean.correction=FALSE)
Dlsp500.Mod01Fit <- fitted(Dlsp500.Mod01, verbose=TRUE)

##0-adj model w/asymmetry:
Dlsp500Asym <- log(Dlsp500adj^2)*(Dlsp500 < 0)
Dlsp500Asym <- glag(Dlsp500Asym, pad.value=0)
Dlsp500.Mod03 <- lgarch(Dlsp500adj, xreg=Dlsp500Asym,
  method="ml", mean.correction=FALSE)
Dlsp500.Mod03Fit <- fitted(Dlsp500.Mod03, verbose=TRUE)

##zeros not replaced (Algo):
Dlsp500.Mod02 <- lgarch(Dlsp500, method="ml",
    mean.correction=FALSE)
Dlsp500.Mod02Fit <- fitted(Dlsp500.Mod02, verbose=TRUE)
Dlsp500.Mod02zhat <- Dlsp500.Mod02Fit[,"zhat"]
Dlsp500.Mod02zhat[ which( Dlsp500.Mod02zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/asymmetry:
Dlsp500.Mod04 <- lgarch(Dlsp500, xreg=Dlsp500Asym,
  method="ml", mean.correction=FALSE)
Dlsp500.Mod04Fit <- fitted(Dlsp500.Mod04, verbose=TRUE)
Dlsp500.Mod04zhat <- Dlsp500.Mod04Fit[,"zhat"]
Dlsp500.Mod04zhat[ which( Dlsp500.Mod04zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/extended asymmetry:
Dlsp500AsymEx <- (Dlsp500 < 0)
Dlsp500AsymEx <- glag(Dlsp500AsymEx, pad.value=0)
Dlsp500.Mod06 <- lgarch(Dlsp500, xreg=cbind(Dlsp500Asym,Dlsp500AsymEx),
  method="ml", mean.correction=FALSE)
Dlsp500.Mod06Fit <- fitted(Dlsp500.Mod06, verbose=TRUE)
Dlsp500.Mod06zhat <- Dlsp500.Mod06Fit[,"zhat"]
Dlsp500.Mod06zhat[ which( Dlsp500.Mod06zhat==0 ) ] <- NA

##latex code:
Dlsp500.latex <- paste("Apple (10 Sep. 1984 -- 23 Aug. 2013): & ",
  ground(var(Dlsp500), digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dlsp500),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox.test(Dlsp500^2)[1,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dlsp500^2)[1,2], digits=2, as.character=TRUE),
  "}$ & ",
  "$\\underset{[", ground(gBox.test(Dlsp500^2)[2,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dlsp500^2)[2,2], digits=2, as.character=TRUE),
  "}$ & ",
  length(Dlsp500), " & ",
  length(Dlsp500.where.zeros), " & ",
  ground(length(Dlsp500.where.zeros)/length(Dlsp500), digits=3,
  as.character=TRUE), " \\", sep="")
Dlsp500.latexMod01 <- paste("SP500: & & 0-adj & ",
  ground(as.numeric(Dlsp500.Mod01$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod01$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod01))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod01$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod01))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlsp500.Mod01Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlsp500.Mod01Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dlsp500.latexMod02 <- paste(" & & Algo & ",
  ground(as.numeric(Dlsp500.Mod02$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod02$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod02))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod02$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod02))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlsp500.Mod02zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlsp500.Mod02zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")
Dlsp500.latexMod03 <- paste("SP500: & & 0-adj & ",
  ground(as.numeric(Dlsp500.Mod03$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod03$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod03))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod03$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod03))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod03$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod03))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlsp500.Mod03Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlsp500.Mod03Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dlsp500.latexMod04 <- paste(" & & Algo & ",
  ground(as.numeric(Dlsp500.Mod04$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod04$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod04))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod04$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod04))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlsp500.Mod04$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlsp500.Mod04))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlsp500.Mod04zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlsp500.Mod04zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")


###########################################################
## 7 OILPRICE
###########################################################

moilp <- read.csv("oilprice.csv")
oilp <- zoo(moilp[,2], order.by=as.Date(moilp[,1], "%d/%m/%Y"))
Dloilp <- diff(log(oilp))*100

##zeros replaced by small non-zero value (0-adj):
Dloilp.where.zeros <- which( Dloilp == 0)
Dloilp.zero.eps <- min(abs(Dloilp[-Dloilp.where.zeros]))
Dloilpadj <- Dloilp
Dloilpadj[Dloilp.where.zeros] <- Dloilp.zero.eps

##0-adj model:
Dloilp.Mod01 <- lgarch(Dloilpadj, method="ml",
    mean.correction=FALSE)
Dloilp.Mod01Fit <- fitted(Dloilp.Mod01, verbose=TRUE)

##0-adj model w/asymmetry:
DloilpAsym <- log(Dloilpadj^2)*(Dloilp < 0)
DloilpAsym <- glag(DloilpAsym, pad.value=0)
Dloilp.Mod03 <- lgarch(Dloilpadj, xreg=DloilpAsym,
  method="ml", mean.correction=FALSE)
Dloilp.Mod03Fit <- fitted(Dloilp.Mod03, verbose=TRUE)

##zeros not replaced (Algo):
Dloilp.Mod02 <- lgarch(Dloilp, method="ml",
    mean.correction=FALSE)
Dloilp.Mod02Fit <- fitted(Dloilp.Mod02, verbose=TRUE)
Dloilp.Mod02zhat <- Dloilp.Mod02Fit[,"zhat"]
Dloilp.Mod02zhat[ which( Dloilp.Mod02zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/asymmetry:
Dloilp.Mod04 <- lgarch(Dloilp, xreg=DloilpAsym,
  method="ml", mean.correction=FALSE)
Dloilp.Mod04Fit <- fitted(Dloilp.Mod04, verbose=TRUE)
Dloilp.Mod04zhat <- Dloilp.Mod04Fit[,"zhat"]
Dloilp.Mod04zhat[ which( Dloilp.Mod04zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/extended asymmetry:
DloilpAsymEx <- (Dloilp < 0)
DloilpAsymEx <- glag(DloilpAsymEx, pad.value=0)
Dloilp.Mod06 <- lgarch(Dloilp, xreg=cbind(DloilpAsym,DloilpAsymEx),
  method="ml", mean.correction=FALSE)
Dloilp.Mod06Fit <- fitted(Dloilp.Mod06, verbose=TRUE)
Dloilp.Mod06zhat <- Dloilp.Mod06Fit[,"zhat"]
Dloilp.Mod06zhat[ which( Dloilp.Mod06zhat==0 ) ] <- NA

##latex code:
Dloilp.latex <- paste("Apple (10 Sep. 1984 -- 23 Aug. 2013): & ",
  ground(var(Dloilp), digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dloilp),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox.test(Dloilp^2)[1,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dloilp^2)[1,2], digits=2, as.character=TRUE),
  "}$ & ",
  "$\\underset{[", ground(gBox.test(Dloilp^2)[2,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dloilp^2)[2,2], digits=2, as.character=TRUE),
  "}$ & ",
  length(Dloilp), " & ",
  length(Dloilp.where.zeros), " & ",
  ground(length(Dloilp.where.zeros)/length(Dloilp), digits=3,
  as.character=TRUE), " \\", sep="")
Dloilp.latexMod01 <- paste("Oil: & & 0-adj & ",
  ground(as.numeric(Dloilp.Mod01$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod01$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod01))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod01$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod01))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dloilp.Mod01Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dloilp.Mod01Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dloilp.latexMod02 <- paste(" & & Algo & ",
  ground(as.numeric(Dloilp.Mod02$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod02$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod02))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod02$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod02))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dloilp.Mod02zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dloilp.Mod02zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")
Dloilp.latexMod03 <- paste("Oil: & & 0-adj & ",
  ground(as.numeric(Dloilp.Mod03$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod03$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod03))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod03$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod03))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod03$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod03))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dloilp.Mod03Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dloilp.Mod03Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dloilp.latexMod04 <- paste(" & & Algo & ",
  ground(as.numeric(Dloilp.Mod04$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod04$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod04))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod04$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod04))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dloilp.Mod04$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dloilp.Mod04))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dloilp.Mod04zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dloilp.Mod04zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")


###########################################################
## 8 GOLD
###########################################################

mgold <- read.csv("table_gold.csv")
gold <- zoo(mgold[,"GoldPM"], order.by=as.Date(mgold[,1], "%Y-%m-%d"))
goldam <- zoo(mgold[,"GoldAM"], order.by=as.Date(mgold[,1], "%Y-%m-%d"))
gold.which.zero <- which(is.na(gold))
gold[gold.which.zero] <- goldam[gold.which.zero]
Dlgold <- diff(log(gold))*100

##zeros replaced by small non-zero value (0-adj):
Dlgold.where.zeros <- which( Dlgold == 0)
Dlgold.zero.eps <- min(abs(Dlgold[-Dlgold.where.zeros]))
Dlgoldadj <- Dlgold
Dlgoldadj[Dlgold.where.zeros] <- Dlgold.zero.eps

##0-adj model:
Dlgold.Mod01 <- lgarch(Dlgoldadj, method="ml",
    mean.correction=FALSE)
Dlgold.Mod01Fit <- fitted(Dlgold.Mod01, verbose=TRUE)

##0-adj model w/asymmetry:
DlgoldAsym <- log(Dlgoldadj^2)*(Dlgold < 0)
DlgoldAsym <- glag(DlgoldAsym, pad.value=0)
Dlgold.Mod03 <- lgarch(Dlgoldadj, xreg=DlgoldAsym,
  method="ml", mean.correction=FALSE)
Dlgold.Mod03Fit <- fitted(Dlgold.Mod03, verbose=TRUE)

##zeros not replaced (Algo):
Dlgold.Mod02 <- lgarch(Dlgold, method="ml",
    mean.correction=FALSE)
Dlgold.Mod02Fit <- fitted(Dlgold.Mod02, verbose=TRUE)
Dlgold.Mod02zhat <- Dlgold.Mod02Fit[,"zhat"]
Dlgold.Mod02zhat[ which( Dlgold.Mod02zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/asymmetry:
Dlgold.Mod04 <- lgarch(Dlgold, xreg=DlgoldAsym,
  method="ml", mean.correction=FALSE)
Dlgold.Mod04Fit <- fitted(Dlgold.Mod04, verbose=TRUE)
Dlgold.Mod04zhat <- Dlgold.Mod04Fit[,"zhat"]
Dlgold.Mod04zhat[ which( Dlgold.Mod04zhat==0 ) ] <- NA

##zeros not replaced (Algo) w/extended asymmetry:
DlgoldAsymEx <- (Dlgold < 0)
DlgoldAsymEx <- glag(DlgoldAsymEx, pad.value=0)
Dlgold.Mod06 <- lgarch(Dlgold, xreg=cbind(DlgoldAsym,DlgoldAsymEx),
  method="ml", mean.correction=FALSE)
Dlgold.Mod06Fit <- fitted(Dlgold.Mod06, verbose=TRUE)
Dlgold.Mod06zhat <- Dlgold.Mod06Fit[,"zhat"]
Dlgold.Mod06zhat[ which( Dlgold.Mod06zhat==0 ) ] <- NA

##latex code:
Dlgold.latex <- paste("Apple (10 Sep. 1984 -- 23 Aug. 2013): & ",
  ground(var(Dlgold), digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dlgold),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox.test(Dlgold^2)[1,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dlgold^2)[1,2], digits=2, as.character=TRUE),
  "}$ & ",
  "$\\underset{[", ground(gBox.test(Dlgold^2)[2,3],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox.test(Dlgold^2)[2,2], digits=2, as.character=TRUE),
  "}$ & ",
  length(Dlgold), " & ",
  length(Dlgold.where.zeros), " & ",
  ground(length(Dlgold.where.zeros)/length(Dlgold), digits=3,
  as.character=TRUE), " \\", sep="")
Dlgold.latexMod01 <- paste("Gold: & & 0-adj & ",
  ground(as.numeric(Dlgold.Mod01$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod01$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod01))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod01$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod01))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlgold.Mod01Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlgold.Mod01Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dlgold.latexMod02 <- paste(" & & Algo & ",
  ground(as.numeric(Dlgold.Mod02$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod02$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod02))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod02$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod02))))[3], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlgold.Mod02zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlgold.Mod02zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")
Dlgold.latexMod03 <- paste("Gold: & & 0-adj & ",
  ground(as.numeric(Dlgold.Mod03$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod03$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod03))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod03$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod03))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod03$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod03))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlgold.Mod03Fit[,"zhat"]^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlgold.Mod03Fit[,"zhat"]^2)[2,2], digits=1, as.char=T),
  "}$ \\", sep="")
Dlgold.latexMod04 <- paste(" & & Algo & ",
  ground(as.numeric(Dlgold.Mod04$par[1]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod04$par[2]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod04))))[2], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod04$par[3]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod04))))[3], digits=3, as.character=TRUE), " & ",
  ground(as.numeric(Dlgold.Mod04$par[4]), digits=3, as.character=TRUE), " & ",
  ground(as.numeric(sqrt(diag(vcov(Dlgold.Mod04))))[4], digits=3, as.character=TRUE),
  " & $\\underset{[",
  ground(gBox.test(Dlgold.Mod04zhat^2)[2,3], digits=2, as.char=T),
  "]}{",
  ground(gBox.test(Dlgold.Mod04zhat^2)[2,2], digits=1, as.char=T),
  "}$ \\vspace{2mm}\\", sep="")


###########################################################
## 9 MAKE DESCRIPTIVE STATISTICS AND ESTIMATES TABLES
###########################################################

##desc.stats.:
tmp <- rbind(Dlapple.latex, Dlekornes.latex, Dleurusd.latex,
  Dlsp500.latex, Dloilp.latex, Dlgold.latex)
rownames(tmp) <- rep("",NROW(tmp))
tmp

##log-garch(1,1) estimates:
rbind(Dlapple.latexMod01, Dlapple.latexMod02,
  Dlekornes.latexMod01, Dlekornes.latexMod02,
  Dleurusd.latexMod01, Dleurusd.latexMod02,
  Dlsp500.latexMod01, Dlsp500.latexMod02,
  Dloilp.latexMod01, Dloilp.latexMod02,
  Dlgold.latexMod01, Dlgold.latexMod02)

##log-garch(1,1) w/asymmetry estimates:
rbind(Dlapple.latexMod03, Dlapple.latexMod04,
  Dlekornes.latexMod03, Dlekornes.latexMod04,
  Dleurusd.latexMod03, Dleurusd.latexMod04,
  Dlsp500.latexMod03, Dlsp500.latexMod04,
  Dloilp.latexMod03, Dloilp.latexMod04,
  Dlgold.latexMod03, Dlgold.latexMod04)

##table of sd descriptives (models 1 and 2):
desc.table <- matrix(NA,24,4)
colnames(desc.table) <- c("Mean", "s2", "Max", "Min")
rownames(desc.table) <- rep(c("0-adj", "Algo", "Diff", "Ratio"), 6)

#apple:
desc.table[1,1] <- mean(Dlapple.Mod01Fit[,"sd"])
desc.table[1,2] <- var(Dlapple.Mod01Fit[,"sd"])
desc.table[1,3] <- max(Dlapple.Mod01Fit[,"sd"])
desc.table[1,4] <- min(Dlapple.Mod01Fit[,"sd"])

desc.table[2,1] <- mean(Dlapple.Mod02Fit[,"sd"])
desc.table[2,2] <- var(Dlapple.Mod02Fit[,"sd"])
desc.table[2,3] <- max(Dlapple.Mod02Fit[,"sd"])
desc.table[2,4] <- min(Dlapple.Mod02Fit[,"sd"])

diff.apple <- Dlapple.Mod02Fit[,"sd"] - Dlapple.Mod01Fit[,"sd"]
desc.table[3,1] <- mean(diff.apple)
desc.table[3,2] <- var(diff.apple)
desc.table[3,3] <- max(diff.apple)
desc.table[3,4] <- min(diff.apple)

ratio.apple <- Dlapple.Mod02Fit[,"sd"]/Dlapple.Mod01Fit[,"sd"]
desc.table[4,1] <- mean(ratio.apple)
desc.table[4,2] <- var(ratio.apple)
desc.table[4,3] <- max(ratio.apple)
desc.table[4,4] <- min(ratio.apple)

#ekornes:
desc.table[5,1] <- mean(Dlekornes.Mod01Fit[,"sd"])
desc.table[5,2] <- var(Dlekornes.Mod01Fit[,"sd"])
desc.table[5,3] <- max(Dlekornes.Mod01Fit[,"sd"])
desc.table[5,4] <- min(Dlekornes.Mod01Fit[,"sd"])

desc.table[6,1] <- mean(Dlekornes.Mod02Fit[,"sd"])
desc.table[6,2] <- var(Dlekornes.Mod02Fit[,"sd"])
desc.table[6,3] <- max(Dlekornes.Mod02Fit[,"sd"])
desc.table[6,4] <- min(Dlekornes.Mod02Fit[,"sd"])

diff.ekornes <- Dlekornes.Mod02Fit[,"sd"] - Dlekornes.Mod01Fit[,"sd"]
desc.table[7,1] <- mean(diff.ekornes)
desc.table[7,2] <- var(diff.ekornes)
desc.table[7,3] <- max(diff.ekornes)
desc.table[7,4] <- min(diff.ekornes)

ratio.ekornes <- Dlekornes.Mod02Fit[,"sd"]/Dlekornes.Mod01Fit[,"sd"]
desc.table[8,1] <- mean(ratio.ekornes)
desc.table[8,2] <- var(ratio.ekornes)
desc.table[8,3] <- max(ratio.ekornes)
desc.table[8,4] <- min(ratio.ekornes)

#eurusd:
desc.table[9,1] <- mean(Dleurusd.Mod01Fit[,"sd"])
desc.table[9,2] <- var(Dleurusd.Mod01Fit[,"sd"])
desc.table[9,3] <- max(Dleurusd.Mod01Fit[,"sd"])
desc.table[9,4] <- min(Dleurusd.Mod01Fit[,"sd"])

desc.table[10,1] <- mean(Dleurusd.Mod02Fit[,"sd"])
desc.table[10,2] <- var(Dleurusd.Mod02Fit[,"sd"])
desc.table[10,3] <- max(Dleurusd.Mod02Fit[,"sd"])
desc.table[10,4] <- min(Dleurusd.Mod02Fit[,"sd"])

diff.eurusd <- Dleurusd.Mod02Fit[,"sd"] - Dleurusd.Mod01Fit[,"sd"]
desc.table[11,1] <- mean(diff.eurusd)
desc.table[11,2] <- var(diff.eurusd)
desc.table[11,3] <- max(diff.eurusd)
desc.table[11,4] <- min(diff.eurusd)

ratio.eurusd <- Dleurusd.Mod02Fit[,"sd"]/Dleurusd.Mod01Fit[,"sd"]
desc.table[12,1] <- mean(ratio.eurusd)
desc.table[12,2] <- var(ratio.eurusd)
desc.table[12,3] <- max(ratio.eurusd)
desc.table[12,4] <- min(ratio.eurusd)

#sp500:
desc.table[13,1] <- mean(Dlsp500.Mod01Fit[,"sd"])
desc.table[13,2] <- var(Dlsp500.Mod01Fit[,"sd"])
desc.table[13,3] <- max(Dlsp500.Mod01Fit[,"sd"])
desc.table[13,4] <- min(Dlsp500.Mod01Fit[,"sd"])

desc.table[14,1] <- mean(Dlsp500.Mod02Fit[,"sd"])
desc.table[14,2] <- var(Dlsp500.Mod02Fit[,"sd"])
desc.table[14,3] <- max(Dlsp500.Mod02Fit[,"sd"])
desc.table[14,4] <- min(Dlsp500.Mod02Fit[,"sd"])

diff.sp500 <- Dlsp500.Mod02Fit[,"sd"] - Dlsp500.Mod01Fit[,"sd"]
desc.table[15,1] <- mean(diff.sp500)
desc.table[15,2] <- var(diff.sp500)
desc.table[15,3] <- max(diff.sp500)
desc.table[15,4] <- min(diff.sp500)

ratio.sp500 <- Dlsp500.Mod02Fit[,"sd"]/Dlsp500.Mod01Fit[,"sd"]
desc.table[16,1] <- mean(ratio.sp500)
desc.table[16,2] <- var(ratio.sp500)
desc.table[16,3] <- max(ratio.sp500)
desc.table[16,4] <- min(ratio.sp500)

#oilprice:
desc.table[17,1] <- mean(Dloilp.Mod01Fit[,"sd"])
desc.table[17,2] <- var(Dloilp.Mod01Fit[,"sd"])
desc.table[17,3] <- max(Dloilp.Mod01Fit[,"sd"])
desc.table[17,4] <- min(Dloilp.Mod01Fit[,"sd"])

desc.table[18,1] <- mean(Dloilp.Mod02Fit[,"sd"])
desc.table[18,2] <- var(Dloilp.Mod02Fit[,"sd"])
desc.table[18,3] <- max(Dloilp.Mod02Fit[,"sd"])
desc.table[18,4] <- min(Dloilp.Mod02Fit[,"sd"])

diff.oilp <- Dloilp.Mod02Fit[,"sd"] - Dloilp.Mod01Fit[,"sd"]
desc.table[19,1] <- mean(diff.oilp)
desc.table[19,2] <- var(diff.oilp)
desc.table[19,3] <- max(diff.oilp)
desc.table[19,4] <- min(diff.oilp)

ratio.oilp <- Dloilp.Mod02Fit[,"sd"]/Dloilp.Mod01Fit[,"sd"]
desc.table[20,1] <- mean(ratio.oilp)
desc.table[20,2] <- var(ratio.oilp)
desc.table[20,3] <- max(ratio.oilp)
desc.table[20,4] <- min(ratio.oilp)

#gold:
desc.table[21,1] <- mean(Dlgold.Mod01Fit[,"sd"])
desc.table[21,2] <- var(Dlgold.Mod01Fit[,"sd"])
desc.table[21,3] <- max(Dlgold.Mod01Fit[,"sd"])
desc.table[21,4] <- min(Dlgold.Mod01Fit[,"sd"])

desc.table[22,1] <- mean(Dlgold.Mod02Fit[,"sd"])
desc.table[22,2] <- var(Dlgold.Mod02Fit[,"sd"])
desc.table[22,3] <- max(Dlgold.Mod02Fit[,"sd"])
desc.table[22,4] <- min(Dlgold.Mod02Fit[,"sd"])

diff.gold <- Dlgold.Mod02Fit[,"sd"] - Dlgold.Mod01Fit[,"sd"]
desc.table[23,1] <- mean(diff.gold)
desc.table[23,2] <- var(diff.gold)
desc.table[23,3] <- max(diff.gold)
desc.table[23,4] <- min(diff.gold)

ratio.gold <- Dlgold.Mod02Fit[,"sd"]/Dlgold.Mod01Fit[,"sd"]
desc.table[24,1] <- mean(ratio.gold)
desc.table[24,2] <- var(ratio.gold)
desc.table[24,3] <- max(ratio.gold)
desc.table[24,4] <- min(ratio.gold)

##table of sd descriptives (models 3 and 4, not in paper!):
desc.table2 <- matrix(NA,24,4)
colnames(desc.table2) <- c("Mean", "s2", "Max", "Min")
rownames(desc.table2) <- rep(c("0-adj", "Algo", "Diff", "Ratio"), 6)

#apple:
desc.table2[1,1] <- mean(Dlapple.Mod03Fit[,"sd"])
desc.table2[1,2] <- var(Dlapple.Mod03Fit[,"sd"])
desc.table2[1,3] <- max(Dlapple.Mod03Fit[,"sd"])
desc.table2[1,4] <- min(Dlapple.Mod03Fit[,"sd"])

desc.table2[2,1] <- mean(Dlapple.Mod04Fit[,"sd"])
desc.table2[2,2] <- var(Dlapple.Mod04Fit[,"sd"])
desc.table2[2,3] <- max(Dlapple.Mod04Fit[,"sd"])
desc.table2[2,4] <- min(Dlapple.Mod04Fit[,"sd"])

diff.apple <- Dlapple.Mod04Fit[,"sd"] - Dlapple.Mod03Fit[,"sd"]
desc.table2[3,1] <- mean(diff.apple)
desc.table2[3,2] <- var(diff.apple)
desc.table2[3,3] <- max(diff.apple)
desc.table2[3,4] <- min(diff.apple)

ratio2.apple <- Dlapple.Mod04Fit[,"sd"]/Dlapple.Mod03Fit[,"sd"]
desc.table2[4,1] <- mean(ratio2.apple)
desc.table2[4,2] <- var(ratio2.apple)
desc.table2[4,3] <- max(ratio2.apple)
desc.table2[4,4] <- min(ratio2.apple)

#ekornes:
desc.table2[5,1] <- mean(Dlekornes.Mod03Fit[,"sd"])
desc.table2[5,2] <- var(Dlekornes.Mod03Fit[,"sd"])
desc.table2[5,3] <- max(Dlekornes.Mod03Fit[,"sd"])
desc.table2[5,4] <- min(Dlekornes.Mod03Fit[,"sd"])

desc.table2[6,1] <- mean(Dlekornes.Mod04Fit[,"sd"])
desc.table2[6,2] <- var(Dlekornes.Mod04Fit[,"sd"])
desc.table2[6,3] <- max(Dlekornes.Mod04Fit[,"sd"])
desc.table2[6,4] <- min(Dlekornes.Mod04Fit[,"sd"])

diff.ekornes <- Dlekornes.Mod04Fit[,"sd"] - Dlekornes.Mod03Fit[,"sd"]
desc.table2[7,1] <- mean(diff.ekornes)
desc.table2[7,2] <- var(diff.ekornes)
desc.table2[7,3] <- max(diff.ekornes)
desc.table2[7,4] <- min(diff.ekornes)

ratio2.ekornes <- Dlekornes.Mod04Fit[,"sd"]/Dlekornes.Mod03Fit[,"sd"]
desc.table2[8,1] <- mean(ratio2.ekornes)
desc.table2[8,2] <- var(ratio2.ekornes)
desc.table2[8,3] <- max(ratio2.ekornes)
desc.table2[8,4] <- min(ratio2.ekornes)

#eurusd:
desc.table2[9,1] <- mean(Dleurusd.Mod03Fit[,"sd"])
desc.table2[9,2] <- var(Dleurusd.Mod03Fit[,"sd"])
desc.table2[9,3] <- max(Dleurusd.Mod03Fit[,"sd"])
desc.table2[9,4] <- min(Dleurusd.Mod03Fit[,"sd"])

desc.table2[10,1] <- mean(Dleurusd.Mod04Fit[,"sd"])
desc.table2[10,2] <- var(Dleurusd.Mod04Fit[,"sd"])
desc.table2[10,3] <- max(Dleurusd.Mod04Fit[,"sd"])
desc.table2[10,4] <- min(Dleurusd.Mod04Fit[,"sd"])

diff.eurusd <- Dleurusd.Mod04Fit[,"sd"] - Dleurusd.Mod03Fit[,"sd"]
desc.table2[11,1] <- mean(diff.eurusd)
desc.table2[11,2] <- var(diff.eurusd)
desc.table2[11,3] <- max(diff.eurusd)
desc.table2[11,4] <- min(diff.eurusd)

ratio2.eurusd <- Dleurusd.Mod04Fit[,"sd"]/Dleurusd.Mod03Fit[,"sd"]
desc.table2[12,1] <- mean(ratio2.eurusd)
desc.table2[12,2] <- var(ratio2.eurusd)
desc.table2[12,3] <- max(ratio2.eurusd)
desc.table2[12,4] <- min(ratio2.eurusd)

#sp500:
desc.table2[13,1] <- mean(Dlsp500.Mod03Fit[,"sd"])
desc.table2[13,2] <- var(Dlsp500.Mod03Fit[,"sd"])
desc.table2[13,3] <- max(Dlsp500.Mod03Fit[,"sd"])
desc.table2[13,4] <- min(Dlsp500.Mod03Fit[,"sd"])

desc.table2[14,1] <- mean(Dlsp500.Mod04Fit[,"sd"])
desc.table2[14,2] <- var(Dlsp500.Mod04Fit[,"sd"])
desc.table2[14,3] <- max(Dlsp500.Mod04Fit[,"sd"])
desc.table2[14,4] <- min(Dlsp500.Mod04Fit[,"sd"])

diff.sp500 <- Dlsp500.Mod04Fit[,"sd"] - Dlsp500.Mod03Fit[,"sd"]
desc.table2[15,1] <- mean(diff.sp500)
desc.table2[15,2] <- var(diff.sp500)
desc.table2[15,3] <- max(diff.sp500)
desc.table2[15,4] <- min(diff.sp500)

ratio2.sp500 <- Dlsp500.Mod04Fit[,"sd"]/Dlsp500.Mod03Fit[,"sd"]
desc.table2[16,1] <- mean(ratio2.sp500)
desc.table2[16,2] <- var(ratio2.sp500)
desc.table2[16,3] <- max(ratio2.sp500)
desc.table2[16,4] <- min(ratio2.sp500)

#oilprice:
desc.table2[17,1] <- mean(Dloilp.Mod03Fit[,"sd"])
desc.table2[17,2] <- var(Dloilp.Mod03Fit[,"sd"])
desc.table2[17,3] <- max(Dloilp.Mod03Fit[,"sd"])
desc.table2[17,4] <- min(Dloilp.Mod03Fit[,"sd"])

desc.table2[18,1] <- mean(Dloilp.Mod04Fit[,"sd"])
desc.table2[18,2] <- var(Dloilp.Mod04Fit[,"sd"])
desc.table2[18,3] <- max(Dloilp.Mod04Fit[,"sd"])
desc.table2[18,4] <- min(Dloilp.Mod04Fit[,"sd"])

diff.oilp <- Dloilp.Mod04Fit[,"sd"] - Dloilp.Mod03Fit[,"sd"]
desc.table2[19,1] <- mean(diff.oilp)
desc.table2[19,2] <- var(diff.oilp)
desc.table2[19,3] <- max(diff.oilp)
desc.table2[19,4] <- min(diff.oilp)

ratio2.oilp <- Dloilp.Mod04Fit[,"sd"]/Dloilp.Mod03Fit[,"sd"]
desc.table2[20,1] <- mean(ratio2.oilp)
desc.table2[20,2] <- var(ratio2.oilp)
desc.table2[20,3] <- max(ratio2.oilp)
desc.table2[20,4] <- min(ratio2.oilp)

#gold:
desc.table2[21,1] <- mean(Dlgold.Mod03Fit[,"sd"])
desc.table2[21,2] <- var(Dlgold.Mod03Fit[,"sd"])
desc.table2[21,3] <- max(Dlgold.Mod03Fit[,"sd"])
desc.table2[21,4] <- min(Dlgold.Mod03Fit[,"sd"])

desc.table2[22,1] <- mean(Dlgold.Mod04Fit[,"sd"])
desc.table2[22,2] <- var(Dlgold.Mod04Fit[,"sd"])
desc.table2[22,3] <- max(Dlgold.Mod04Fit[,"sd"])
desc.table2[22,4] <- min(Dlgold.Mod04Fit[,"sd"])

diff.gold <- Dlgold.Mod04Fit[,"sd"] - Dlgold.Mod03Fit[,"sd"]
desc.table2[23,1] <- mean(diff.gold)
desc.table2[23,2] <- var(diff.gold)
desc.table2[23,3] <- max(diff.gold)
desc.table2[23,4] <- min(diff.gold)

ratio2.gold <- Dlgold.Mod04Fit[,"sd"]/Dlgold.Mod03Fit[,"sd"]
desc.table2[24,1] <- mean(ratio2.gold)
desc.table2[24,2] <- var(ratio2.gold)
desc.table2[24,3] <- max(ratio2.gold)
desc.table2[24,4] <- min(ratio2.gold)


###########################################################
## 10 MAKE FIGURES
###########################################################

#figure of returns (not in paper!):
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.05, 0.05))
layout(t(matrix(1:6, 2, 3)))
plot(Dlapple, col="blue", ylab="Apple", xlab="", main="")
plot(Dlekornes, col="blue", ylab="Ekornes", xlab="", main="")
plot(Dleurusd, col="blue", ylab="EUR/USD", xlab="", main="")
plot(Dlsp500, col="blue", ylab="SP500", xlab="", main="")
plot(Dloilp, col="blue", ylab="Oil", xlab="", main="")
plot(Dlgold, col="blue", ylab="Gold", xlab="", main="")
par(def.par)

#figure of standard deviations (not in paper!):
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.05, 0.05))
layout(t(matrix(1:6, 2, 3)))
tmp.apple <- cbind(Dlapple.Mod02Fit[,"sd"],
  Dlapple.Mod01Fit[,"sd"])
colnames(tmp.apple) <- c("Algo", "0-adj")
plot(tmp.apple, plot.type="single", col=c("blue", "red"),
  lty=c("solid","dashed"), ylab="Apple", xlab="",
  main="")
legend("topright",c("Algo", "0-adj"), lty=c("solid","dashed"),
  lwd=2, col=c("blue", "red"), bty="n", cex=0.8)
tmp.ekornes <- cbind(Dlekornes.Mod02Fit[,"sd"],
  Dlekornes.Mod01Fit[,"sd"])
colnames(tmp.ekornes) <- c("Algo", "0-adj")
plot(tmp.ekornes, plot.type="single", col=c("blue", "red"),
  lty=c("solid","dashed"), ylab="Ekornes", xlab="",
  main="")
legend("topright",c("Algo", "0-adj"), lty=c("solid","dashed"),
  lwd=2, col=c("blue", "red"), bty="n", cex=0.8)
tmp.eurusd <- cbind(Dleurusd.Mod02Fit[,"sd"],
  Dleurusd.Mod01Fit[,"sd"])
colnames(tmp.eurusd) <- c("Algo", "0-adj")
plot(tmp.eurusd, plot.type="single", col=c("blue", "red"),
  lty=c("solid","dashed"), ylab="EUR/USD", xlab="",
  main="")
legend("topright",c("Algo", "0-adj"), lty=c("solid","dashed"),
  lwd=2, col=c("blue", "red"), bty="n", cex=0.8)
tmp.sp500 <- cbind(Dlsp500.Mod02Fit[,"sd"],
  Dlsp500.Mod01Fit[,"sd"])
colnames(tmp.sp500) <- c("Algo", "0-adj")
plot(tmp.sp500, plot.type="single", col=c("blue", "red"),
  lty=c("solid","dashed"), ylab="SP500", xlab="",
  main="")
legend("topright",c("Algo", "0-adj"), lty=c("solid","dashed"),
  lwd=2, col=c("blue", "red"), bty="n", cex=0.8)
tmp.oilp <- cbind(Dloilp.Mod02Fit[,"sd"],
  Dloilp.Mod01Fit[,"sd"])
colnames(tmp.oilp) <- c("Algo", "0-adj")
plot(tmp.oilp, plot.type="single", col=c("blue", "red"),
  lty=c("solid","dashed"), ylab="Oil", xlab="",
  main="")
legend("topright",c("Algo", "0-adj"), lty=c("solid","dashed"),
  lwd=2, col=c("blue", "red"), bty="n", cex=0.8)
tmp.gold <- cbind(Dlgold.Mod02Fit[,"sd"],
  Dlgold.Mod01Fit[,"sd"])
colnames(tmp.gold) <- c("Algo", "0-adj")
plot(tmp.gold, plot.type="single", col=c("blue", "red"),
  lty=c("solid","dashed"), ylab="Gold", xlab="",
  main="", )
legend("topright",c("Algo", "0-adj"), lty=c("solid","dashed"),
  lwd=2, col=c("blue", "red"), bty="n", cex=0.8)
par(def.par)

#figure of SD difference (not in paper!):
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.1, 0.05))
layout(t(matrix(1:6, 2, 3)))
plot(diff.apple, col=c("blue"), ylab="Apple", xlab="",
  main="", lwd=2) #ylim=c(-1,3)
abline(0,0, col="grey", lty="solid")
plot(diff.ekornes, col=c("blue"), ylab="Ekornes", xlab="",
  main="", lwd=2)
abline(0,0, col="grey", lty="solid")
plot(diff.eurusd, col=c("blue"), ylab="eurusd", xlab="",
  main="", lwd=2)
abline(0,0, col="grey", lty="solid")
plot(diff.sp500, col=c("blue"), ylab="SP500", xlab="",
  main="", lwd=2)
abline(0,0, col="grey", lty="solid")
plot(diff.oilp, col=c("blue"), ylab="Oil", xlab="",
  main="", lwd=2)
abline(0,0, col="grey", lty="solid")
plot(diff.gold, col=c("blue"), ylab="Gold", xlab="",
  main="", lwd=2)
abline(0,0, col="grey", lty="solid")
par(def.par)

#figure of SD ratios:
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.05, 0.05))
layout(t(matrix(1:6, 2, 3)))
plot(ratio.apple, col=c("blue"), ylab="Apple", xlab="",
  main="", lwd=2) #ylim=c(0.5,2.6)
abline(1,0, col="grey", lty="solid")
plot(ratio.ekornes, col=c("blue"), ylab="Ekornes", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
plot(ratio.eurusd, col=c("blue"), ylab="eurusd", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
plot(ratio.sp500, col=c("blue"), ylab="SP500", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
plot(ratio.oilp, col=c("blue"), ylab="Oil", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
plot(ratio.gold, col=c("blue"), ylab="Gold", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
par(def.par)

#figure of SD ratios no. 2 (not in paper!):
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.05, 0.05))
layout(t(matrix(1:6, 2, 3)))
plot(ratio2.apple, col=c("blue"), ylab="Apple", xlab="",
  main="", lwd=2) #ylim=c(0.5,2.6)
abline(1,0, col="grey", lty="solid")
plot(ratio2.ekornes, col=c("blue"), ylab="Ekornes", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
plot(ratio2.eurusd, col=c("blue"), ylab="eurusd", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
plot(ratio2.sp500, col=c("blue"), ylab="SP500", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
plot(ratio2.oilp, col=c("blue"), ylab="Oil", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
plot(ratio2.gold, col=c("blue"), ylab="Gold", xlab="",
  main="", lwd=2)
abline(1,0, col="grey", lty="solid")
par(def.par)


###########################################################
## 11 CHECK WHETHER MEAN-CORRECTION REMOVES ARCH
###########################################################

##this analysis is not contained in the paper, but it is
## mentioned towards the end of Section 4.1.

library(gets)

meanDlapple <- arx(Dlapple, mc=1, ar=1)
DlappleEps <- residuals(meanDlapple)
DlappleEpsAsym <- log(DlappleEps^2)*(Dlapple < 0)
DlappleEpsAsym <- glag(DlappleEpsAsym, pad.value=0)
DlappleEpsAsymExt <- (Dlapple < 0)
DlappleEpsAsymExt <- glag(DlappleEpsAsymExt, pad.value=0)
DlappleEpsAsymExt <- DlappleEpsAsymExt[-1]
DlappleEpsGarch <- lgarch(DlappleEps, method="ml")
#DlappleEpsGarch <- lgarch(DlappleEps,
#  xreg=DlappleEpsAsym, method="ml")
#DlappleEpsGarch <- lgarch(DlappleEps,
#  xreg=cbind(DlappleEpsAsym,DlappleEpsGarch), method="ml")
DlappleEpsGarchFit <- fitted(DlappleEpsGarch, verbose=TRUE)
DlappleEpszhat <- DlappleEpsGarchFit[,"zhat"]
gBox.test(DlappleEpszhat)
gBox.test(DlappleEpszhat^2)

meanDlekornes <- arx(Dlekornes, mc=1, ar=1)
DlekornesEps <- residuals(meanDlekornes)
DlekornesEpsAsym <- log(DlekornesEps^2)*(Dlekornes < 0)
DlekornesEpsAsym <- glag(DlekornesEpsAsym, pad.value=0)
DlekornesEpsAsymExt <- (Dlekornes < 0)
DlekornesEpsAsymExt <- glag(DlekornesEpsAsymExt, pad.value=0)
DlekornesEpsAsymExt <- DlekornesEpsAsymExt[-1]
#DlekornesEpsGarch <- lgarch(DlekornesEps, method="ml")
DlekornesEpsGarch <- lgarch(DlekornesEps,
  xreg=DlekornesEpsAsym, method="ml")
#DlekornesEpsGarch <- lgarch(DlekornesEps,
#  xreg=cbind(DlekornesEpsAsym,DlekornesEpsGarch), method="ml")
DlekornesEpsGarchFit <- fitted(DlekornesEpsGarch, verbose=TRUE)
DlekornesEpszhat <- DlekornesEpsGarchFit[,"zhat"]
gBox.test(DlekornesEpszhat)
gBox.test(DlekornesEpszhat^2)

meanDleurusd <- arx(Dleurusd, mc=1, ar=1)
DleurusdEps <- residuals(meanDleurusd)
DleurusdEpsAsym <- log(DleurusdEps^2)*(Dleurusd < 0)
DleurusdEpsAsym <- glag(DleurusdEpsAsym, pad.value=0)
DleurusdEpsAsymExt <- (Dleurusd < 0)
DleurusdEpsAsymExt <- glag(DleurusdEpsAsymExt, pad.value=0)
DleurusdEpsAsymExt <- DleurusdEpsAsymExt[-1]
#DleurusdEpsGarch <- lgarch(DleurusdEps, method="ml")
DleurusdEpsGarch <- lgarch(DleurusdEps,
  xreg=DleurusdEpsAsym, method="ml")
#DleurusdEpsGarch <- lgarch(DleurusdEps,
#  xreg=cbind(DleurusdEpsAsym,DleurusdEpsGarch), method="ml")
DleurusdEpsGarchFit <- fitted(DleurusdEpsGarch, verbose=TRUE)
DleurusdEpszhat <- DleurusdEpsGarchFit[,"zhat"]
gBox.test(DleurusdEpszhat)
gBox.test(DleurusdEpszhat^2)

meanDlsp500 <- arx(Dlsp500, mc=1, ar=1)
Dlsp500Eps <- residuals(meanDlsp500)
Dlsp500EpsAsym <- log(Dlsp500Eps^2)*(Dlsp500 < 0)
Dlsp500EpsAsym <- glag(Dlsp500EpsAsym, pad.value=0)
Dlsp500EpsAsymExt <- (Dlsp500 < 0)
Dlsp500EpsAsymExt <- glag(Dlsp500EpsAsymExt, pad.value=0)
Dlsp500EpsAsymExt <- Dlsp500EpsAsymExt[-1]
#Dlsp500EpsGarch <- lgarch(Dlsp500Eps, method="ml")
Dlsp500EpsGarch <- lgarch(Dlsp500Eps,
  xreg=Dlsp500EpsAsym, method="ml")
#Dlsp500EpsGarch <- lgarch(Dlsp500Eps,
#  xreg=cbind(Dlsp500EpsAsym,Dlsp500EpsGarch), method="ml")
Dlsp500EpsGarchFit <- fitted(Dlsp500EpsGarch, verbose=TRUE)
Dlsp500Epszhat <- Dlsp500EpsGarchFit[,"zhat"]
gBox.test(Dlsp500Epszhat)
gBox.test(Dlsp500Epszhat^2)

meanDloilp <- arx(Dloilp, mc=1, ar=1)
DloilpEps <- residuals(meanDloilp)
DloilpEpsAsym <- log(DloilpEps^2)*(Dloilp < 0)
DloilpEpsAsym <- glag(DloilpEpsAsym, pad.value=0)
DloilpEpsAsymExt <- (Dloilp < 0)
DloilpEpsAsymExt <- glag(DloilpEpsAsymExt, pad.value=0)
DloilpEpsAsymExt <- DloilpEpsAsymExt[-1]
#DloilpEpsGarch <- lgarch(DloilpEps, method="ml")
DloilpEpsGarch <- lgarch(DloilpEps,
  xreg=DloilpEpsAsym, method="ml")
#DloilpEpsGarch <- lgarch(DloilpEps,
#  xreg=cbind(DloilpEpsAsym,DloilpEpsGarch), method="ml")
DloilpEpsGarchFit <- fitted(DloilpEpsGarch, verbose=TRUE)
DloilpEpszhat <- DloilpEpsGarchFit[,"zhat"]
gBox.test(DloilpEpszhat)
gBox.test(DloilpEpszhat^2)

meanDlgold <- arx(Dlgold, mc=1, ar=1)
DlgoldEps <- residuals(meanDlgold)
DlgoldEpsAsym <- log(DlgoldEps^2)*(Dlgold < 0)
DlgoldEpsAsym <- glag(DlgoldEpsAsym, pad.value=0)
DlgoldEpsAsymExt <- (Dlgold < 0)
DlgoldEpsAsymExt <- glag(DlgoldEpsAsymExt, pad.value=0)
DlgoldEpsAsymExt <- DlgoldEpsAsymExt[-1]
#DlgoldEpsGarch <- lgarch(DlgoldEps, method="ml")
DlgoldEpsGarch <- lgarch(DlgoldEps,
  xreg=DlgoldEpsAsym, method="ml")
#DlgoldEpsGarch <- lgarch(DlgoldEps,
#  xreg=cbind(DlgoldEpsAsym,DlgoldEpsGarch), method="ml")
DlgoldEpsGarchFit <- fitted(DlgoldEpsGarch, verbose=TRUE)
DlgoldEpszhat <- DlgoldEpsGarchFit[,"zhat"]
gBox.test(DlgoldEpszhat)
gBox.test(DlgoldEpszhat^2)
